@@ -8,32 +8,31 @@
  * Copyright 1999-2003 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <string.h>
 
+#include "libedataserver/e-memory.h"
+
 #include "camel-folder.h"
 #include "camel-exception.h"
 #include "camel-store.h"
 #include "camel-mime-message.h"
 #include "camel-debug.h"
-#include "libedataserver/e-memory.h"
 #include "camel-operation.h"
 #include "camel-session.h"
 #include "camel-filter-driver.h"
@@ -41,7 +40,7 @@
 #include "camel-vtrash-folder.h"
 #include "camel-i18n.h"
 
-#define d(x) 
+#define d(x)
 #define w(x)
 
 extern int camel_verbose_debug;
@@ -296,11 +295,7 @@ camel_folder_refresh_info (CamelFolder *folder, CamelException *ex)
 {
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
 
-	CAMEL_FOLDER_LOCK(folder, lock);
-
 	CF_CLASS (folder)->refresh_info (folder, ex);
-
-	CAMEL_FOLDER_UNLOCK(folder, lock);
 }
 
 static int
