@@ -9,15 +9,15 @@
  * Copyright 1999-2003 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU General Public 
+ * modify it under the terms of version 2 of the GNU Lesser General Public 
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -32,14 +32,13 @@
 #include <sys/stat.h>
 
 #include "camel-debug.h"
-
+#include "camel-exception.h"
+#include "camel-folder.h"
+#include "camel-i18n.h"
+#include "camel-private.h"
 #include "camel-session.h"
 #include "camel-store.h"
-#include "camel-folder.h"
 #include "camel-vtrash-folder.h"
-#include "camel-exception.h"
-#include "camel-private.h"
-#include "camel-i18n.h"
 
 #define d(x)
 #define w(x)
@@ -287,6 +286,16 @@ camel_store_get_folder (CamelStore *store, const char *folder_name, guint32 flag
 			camel_object_trigger_event(store, "folder_opened", folder);
 	}
 
+	if (camel_debug_start(":store")) {
+		char *u = camel_url_to_string(((CamelService *)store)->url, CAMEL_URL_HIDE_PASSWORD);
+
+		printf("CamelStore('%s'):get_folder('%s', %d) = %p\n", u, folder_name, flags, folder);
+		if (ex && ex->id)
+			printf("  failed: '%s'\n", ex->desc);
+		g_free(u);
+		camel_debug_end();
+	}
+
 	return folder;
 }
 
@@ -429,7 +438,7 @@ camel_store_rename_folder (CamelStore *store, const char *old_namein, const char
 {
 	CamelFolder *folder;
 	int i, oldlen, namelen;
-	GPtrArray *folders;
+	GPtrArray *folders = NULL;
 	char *old_name;
 
 	d(printf("store rename folder %s '%s' '%s'\n", ((CamelService *)store)->url->protocol, old_name, new_name));
@@ -476,7 +485,7 @@ camel_store_rename_folder (CamelStore *store, const char *old_namein, const char
 	CS_CLASS (store)->rename_folder (store, old_name, new_name, ex);
 
 	/* If it worked, update all open folders/unlock them */
-	if (!camel_exception_is_set(ex)) {
+	if (folders && !camel_exception_is_set(ex)) {
 		guint32 flags = CAMEL_STORE_FOLDER_INFO_RECURSIVE;
 		CamelRenameInfo reninfo;
 
@@ -875,12 +884,6 @@ folder_info_cmp (const void *ap, const void *bp)
 	return strcmp (a->full_name, b->full_name);
 }
 
-static void
-free_name(void *key, void *data, void *user)
-{
-	g_free(key);
-}
-
 /**
  * camel_folder_info_build:
  * @folders: an array of #CamelFolderInfo
@@ -908,9 +911,9 @@ camel_folder_info_build (GPtrArray *folders, const char *namespace,
 {
 	CamelFolderInfo *fi, *pfi, *top = NULL, *tail = NULL;
 	GHashTable *hash;
-	char *name, *p, *pname;
+	char *p, *pname;
 	int i, nlen;
-	
+
 	if (!namespace)
 		namespace = "";
 	nlen = strlen (namespace);
@@ -921,30 +924,16 @@ camel_folder_info_build (GPtrArray *folders, const char *namespace,
 	hash = g_hash_table_new (g_str_hash, g_str_equal);
 	for (i = 0; i < folders->len; i++) {
 		fi = folders->pdata[i];
-		if (!strncmp (namespace, fi->full_name, nlen))
-			name = fi->full_name + nlen;
-		else
-			name = fi->full_name;
-		if (*name == separator)
-			name++;
-
-		g_hash_table_insert (hash, g_strdup(name), fi);
+		g_hash_table_insert (hash, fi->full_name, fi);
 	}
 	
 	/* Now find parents. */
 	for (i = 0; i < folders->len; i++) {
 		fi = folders->pdata[i];
-		if (!strncmp (namespace, fi->full_name, nlen))
-			name = fi->full_name + nlen;
-		else
-			name = fi->full_name;
-		if (*name == separator)
-			name++;
-
-		p = strrchr (name, separator);
-		if (p) {
-			pname = g_strndup (name, p - name);
-			pfi = g_hash_table_lookup (hash, pname);
+		if (!strncmp (namespace, fi->full_name, nlen)
+		    && (p = strrchr(fi->full_name+nlen, separator))) {
+			pname = g_strndup(fi->full_name, p - fi->full_name);
+			pfi = g_hash_table_lookup(hash, pname);
 			if (pfi) {
 				g_free (pname);
 			} else {
@@ -994,7 +983,6 @@ camel_folder_info_build (GPtrArray *folders, const char *namespace,
 		} else if (!top)
 			top = fi;
 	}
-	g_hash_table_foreach(hash, free_name, NULL);
 	g_hash_table_destroy (hash);
 
 	/* Link together the top-level folders */
@@ -1290,8 +1278,10 @@ void camel_isubscribe_subscribe(CamelStore *store, const char *folder_name, Came
 {
 	CamelISubscribe *iface = camel_object_get_interface(store, camel_isubscribe_get_type());
 
-	if (iface && iface->subscribe)
-		return iface->subscribe(store, folder_name, ex);
+	if (iface && iface->subscribe) {
+		iface->subscribe(store, folder_name, ex);
+		return;
+	}
 
 	g_warning("Trying to invoke unimplemented subscribe method on a store");
 }
@@ -1300,8 +1290,10 @@ void camel_isubscribe_unsubscribe(CamelStore *store, const char *folder_name, Ca
 {
 	CamelISubscribe *iface = camel_object_get_interface(store, camel_isubscribe_get_type());
 
-	if (iface && iface->unsubscribe)
-		return iface->unsubscribe(store, folder_name, ex);
+	if (iface && iface->unsubscribe) {
+		iface->unsubscribe(store, folder_name, ex);
+		return;
+	}
 
 	g_warning("Trying to invoke unimplemented unsubscribe method on a store");
 }
