@@ -26,25 +26,25 @@
 
 #include <string.h>
 
-#include "camel-exception.h"
-#include "camel-vee-folder.h"
-#include "camel-store.h"
-#include "camel-mime-message.h"
-#include "camel-folder-search.h"
-#include "camel-vee-summary.h"
-
-#include "camel-session.h"
-#include "camel-vee-store.h"	/* for open flags */
-#include "camel-private.h"
-#include "camel-debug.h"
-#include "camel-i18n.h"
-
 #include "libedataserver/md5-utils.h"
 
 #if defined (DOEPOOLV) || defined (DOESTRV)
 #include "libedataserver/e-memory.h"
 #endif
 
+#include "camel-debug.h"
+#include "camel-exception.h"
+#include "camel-folder-search.h"
+#include "camel-i18n.h"
+#include "camel-mime-message.h"
+#include "camel-private.h"
+#include "camel-session.h"
+#include "camel-store.h"
+#include "camel-vee-folder.h"
+#include "camel-vee-store.h"	/* for open flags */
+#include "camel-vee-summary.h"
+
+
 #define d(x) 
 #define dd(x) (camel_debug("vfolder")?(x):0)
 
@@ -535,7 +535,7 @@ vee_search_by_expression(CamelFolder *folder, const char *expression, CamelExcep
 	CAMEL_VEE_FOLDER_LOCK(vf, subfolder_lock);
 	
 	if (vf != folder_unmatched)
-		expr = g_strdup_printf ("(and %s %s)", vf->expression, expression);
+		expr = g_strdup_printf ("(and %s %s)", vf->expression ? vf->expression : "", expression);
 	else
 		expr = g_strdup (expression);
 	
@@ -587,7 +587,7 @@ vee_search_by_uids(CamelFolder *folder, const char *expression, GPtrArray *uids,
 
 	CAMEL_VEE_FOLDER_LOCK(vf, subfolder_lock);
 
-	expr = g_strdup_printf("(and %s %s)", vf->expression, expression);
+	expr = g_strdup_printf("(and %s %s)", vf->expression ? vf->expression : "", expression);
 	node = p->folders;
 	while (node) {
 		CamelFolder *f = node->data;
