@@ -5,15 +5,15 @@
  *	    Jacob Berkman <jacob@ximian.com>
  *
  * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU General Public 
+ * modify it under the terms of version 2 of the GNU Lesser General Public 
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -24,6 +24,7 @@
 
 #include <string.h> /* memset() */
 #include <stdlib.h> /* alloca() */
+#include <glib.h>
 
 #define s(x)			/* strv debug */
 #define p(x)   /* poolv debug */
@@ -99,8 +100,6 @@ checkmem(void *p)
 
 /* mempool class */
 
-#define STRUCT_ALIGN (4)
-
 typedef struct _MemChunkFreeNode {
 	struct _MemChunkFreeNode *next;
 	unsigned int atoms;
@@ -356,14 +355,14 @@ typedef struct _MemPoolNode {
 	struct _MemPoolNode *next;
 
 	int free;
-	char data[1];
 } MemPoolNode;
 
 typedef struct _MemPoolThresholdNode {
 	struct _MemPoolThresholdNode *next;
-	char data[1];
 } MemPoolThresholdNode;
 
+#define ALIGNED_SIZEOF(t)	((sizeof (t) + G_MEM_ALIGN - 1) & -G_MEM_ALIGN)
+
 typedef struct _EMemPool {
 	int blocksize;
 	int threshold;
@@ -424,7 +423,7 @@ MemPool *e_mempool_new(int blocksize, int threshold, EMemPoolFlags flags)
 	switch (flags & E_MEMPOOL_ALIGN_MASK) {
 	case E_MEMPOOL_ALIGN_STRUCT:
 	default:
-		pool->align = STRUCT_ALIGN-1;
+		pool->align = G_MEM_ALIGN-1;
 		break;
 	case E_MEMPOOL_ALIGN_WORD:
 		pool->align = 2-1;
@@ -450,27 +449,27 @@ void *e_mempool_alloc(MemPool *pool, register int size)
 	if (size>=pool->threshold) {
 		MemPoolThresholdNode *n;
 
-		n = g_malloc(sizeof(*n) - sizeof(char) + size);
+		n = g_malloc(ALIGNED_SIZEOF(*n) + size);
 		n->next = pool->threshold_blocks;
 		pool->threshold_blocks = n;
-		return &n->data[0];
+		return (char *) n + ALIGNED_SIZEOF(*n);
 	} else {
 		register MemPoolNode *n;
 
 		n = pool->blocks;
 		if (n && n->free >= size) {
 			n->free -= size;
-			return &n->data[n->free];
+			return (char *) n + ALIGNED_SIZEOF(*n) + n->free;
 		}
 
 		/* maybe we could do some sort of the free blocks based on size, but
 		   it doubt its worth it at all */
 
-		n = g_malloc(sizeof(*n) - sizeof(char) + pool->blocksize);
+		n = g_malloc(ALIGNED_SIZEOF(*n) + pool->blocksize);
 		n->next = pool->blocks;
 		pool->blocks = n;
 		n->free = pool->blocksize - size;
-		return &n->data[n->free];
+		return (char *) n + ALIGNED_SIZEOF(*n) + n->free;
 	}
 }
 
@@ -535,7 +534,13 @@ void e_mempool_destroy(MemPool *pool)
 {
 	if (pool) {
 		e_mempool_flush(pool, 1);
+#ifdef G_THREADS_ENABLED
+		g_static_mutex_lock(&mempool_mutex);
+#endif
 		e_memchunk_free(mempool_memchunk, pool);
+#ifdef G_THREADS_ENABLED
+		g_static_mutex_unlock(&mempool_mutex);
+#endif
 	}
 }
 
