@@ -51,7 +51,6 @@
 #include <gtk/gtkdialog.h>
 #include <gtk/gtklabel.h>
 
-#include <dirent.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -59,9 +58,6 @@
 #define ADS_UF_DONT_EXPIRE_PASSWORD 0x10000
 #define ONE_HUNDRED_NANOSECOND 0.000000100
 #define SECONDS_IN_DAY 86400
-#define PASSWD_EXPIRY_NOTIFICATION_PERIOD 7
-#define FILENAME CONNECTOR_GLADEDIR "/exchange-passwd-expiry.glade"
-#define ROOTNODE "passwd_exp_dialog"
 
 struct _ExchangeAccountPrivate {
 	E2kContext *ctx;
@@ -70,7 +66,8 @@ struct _ExchangeAccountPrivate {
 	ExchangeFolderSize *fsize;
 
 	GMutex *connect_lock;
-	gboolean connecting, connected, account_online;
+	gboolean connecting, connected; 
+	int account_online;
 
 	GPtrArray *hierarchies;
 	GHashTable *hierarchies_by_folder, *foreign_hierarchies;
@@ -160,7 +157,7 @@ init (GObject *object)
 	account->priv->folders = g_hash_table_new (g_str_hash, g_str_equal);
 	account->priv->fresh_folders = NULL;
 	account->priv->discover_data_lock = g_mutex_new ();
-	account->priv->account_online = FALSE;
+	account->priv->account_online = UNSUPPORTED_MODE;
 	account->priv->nt_domain = NULL;
 	account->priv->fsize = exchange_folder_size_new ();
 }
@@ -341,7 +338,7 @@ exchange_account_rescan_tree (ExchangeAccount *account)
 				     toplevel);
 
 		exchange_hierarchy_scan_subtree (account->priv->hierarchies->pdata[i],
-						toplevel, !account->priv->account_online);
+						toplevel, account->priv->account_online);
 		exchange_hierarchy_rescan (account->priv->hierarchies->pdata[i]);
 	}
 }
@@ -362,7 +359,6 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 	/* This makes the cleanup easier. We just unref it each time
 	 * we find it in account->priv->folders.
 	 */
-	d(printf ("adding folder : %s\n", e_folder_get_name (folder)));
 	key = (char *) e_folder_exchange_get_path (folder);
 	if (!g_hash_table_lookup (account->priv->folders, key)) {
 		/* Avoid dupilcations since the user could add a folder as
@@ -371,7 +367,6 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 		g_hash_table_insert (account->priv->folders,
 				     key,
 				     folder);
-		d(printf ("added path : %s\n", key));
 		table_updated = 1;
 	}
 
@@ -390,7 +385,6 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 		g_hash_table_insert (account->priv->folders,
 				     key,
 				     folder);
-		d(printf ("added phy path : %s\n", key));
 		table_updated = 1;
 	}
 
@@ -403,7 +397,6 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 		g_hash_table_insert (account->priv->folders,
 				     key,
 				     folder);
-		d(printf ("added int path : %s\n", key));
 		table_updated = 1;
 	}
 
@@ -536,7 +529,7 @@ exchange_account_remove_folder (ExchangeAccount *account, const char *path)
 
 	if (!get_folder (account, path, &folder, &hier))
 		return EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST;
-	
+
 	int_uri = e_folder_exchange_get_internal_uri (folder);
 
 	if (g_hash_table_find (account->priv->standard_uris, 
@@ -782,7 +775,7 @@ exchange_account_open_folder (ExchangeAccount *account, const char *path)
 {
 	ExchangeHierarchy *hier;
 	EFolder *folder;
-	int offline;
+	int mode;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), 
 				EXCHANGE_ACCOUNT_FOLDER_GENERIC_ERROR);
@@ -790,8 +783,8 @@ exchange_account_open_folder (ExchangeAccount *account, const char *path)
 	if (!get_folder (account, path, &folder, &hier))
 		return EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST;
 
-	exchange_account_is_offline (account, &offline);
-	if (offline == ONLINE_MODE && !account->priv->connected &&
+	exchange_account_is_offline (account, &mode);
+	if (mode == ONLINE_MODE && !account->priv->connected &&
 	    hier == (ExchangeHierarchy *)account->priv->hierarchies->pdata[0] &&
 	    folder == hier->toplevel) {
 		/* The shell is asking us to open the personal folders
@@ -801,7 +794,7 @@ exchange_account_open_folder (ExchangeAccount *account, const char *path)
 		return EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST;
 	}		
 	
-	return exchange_hierarchy_scan_subtree (hier, folder, (offline == OFFLINE_MODE));
+	return exchange_hierarchy_scan_subtree (hier, folder, mode);
 }
 
 ExchangeAccountFolderResult
@@ -1020,58 +1013,8 @@ is_password_expired (ExchangeAccount *account, E2kAutoconfig *ac)
 	return (result == E2K_KERBEROS_PASSWORD_EXPIRED);
 }
 #endif
-#if 0
-static void
-change_passwd_cb (GtkWidget *button, ExchangeAccount *account)
-{
-	char *current_passwd, *new_passwd;
 
-	gtk_widget_hide (gtk_widget_get_toplevel(button));
-	current_passwd = exchange_account_get_password (account);
-	new_passwd = exchange_get_new_password (current_passwd, TRUE);
-	exchange_account_set_password (account, current_passwd, new_passwd);
-	g_free (current_passwd);
-	g_free (new_passwd);
-}
-#endif
-
-static void
-display_passwd_expiry_message (int max_passwd_age, ExchangeAccount *account)
-{
-	GladeXML *xml;
-	GtkWidget *top_widget, *change_passwd_button;
-	GtkResponseType response;
-	GtkLabel *warning_msg_label;
-	char *passwd_expiry_msg = 
-		g_strdup_printf ("Your password will expire in next %d days\n",
-				  max_passwd_age);
-
-	xml = glade_xml_new (FILENAME, ROOTNODE, NULL);
-	g_return_if_fail (xml != NULL);
-	top_widget = glade_xml_get_widget (xml, ROOTNODE);
-	g_return_if_fail (top_widget != NULL);
-
-	warning_msg_label = GTK_LABEL (glade_xml_get_widget (xml, 
-						"passwd_exp_label"));
-	gtk_label_set_text (warning_msg_label, passwd_expiry_msg);
-
-	change_passwd_button = glade_xml_get_widget (xml, 
-						"change_passwd_button");
-	gtk_widget_set_sensitive (change_passwd_button, TRUE);
-/*
-	g_signal_connect (change_passwd_button, 
-			  "clicked", 
-			  G_CALLBACK (change_passwd_cb), 
-			  account);
-*/
-	response = gtk_dialog_run (GTK_DIALOG (top_widget));
-
-	gtk_widget_destroy (top_widget);
-	g_object_unref (xml);
-	g_free (passwd_expiry_msg);
-}
-
-static void
+static int
 find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 {
 	double max_pwd_age = 0;
@@ -1081,7 +1024,7 @@ find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 
 	/* If user has not selected password expiry warning option, return */
 	if (account->priv->passwd_exp_warn_period == -1)
-		return;
+		return -1;
 
 	/* Check for password expiry period */ 
 	/* This needs to be invoked after is_password_expired(), i.e., 
@@ -1098,10 +1041,11 @@ find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 					      &entry); 
 	e2k_operation_free (&gcop);
 	if (gcstatus != E2K_GLOBAL_CATALOG_OK) 
-		return;
+		return -1;
        
-	if (entry->user_account_control & ADS_UF_DONT_EXPIRE_PASSWORD) 
-		return;         /* Password is not set to expire */
+	if (entry->user_account_control & ADS_UF_DONT_EXPIRE_PASSWORD) {
+		return -1;         /* Password is not set to expire */
+	}
 
 	/* Here we don't check not setting the password and expired password */ 
 	/* Check for the maximum password age set */
@@ -1116,10 +1060,10 @@ find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 		( max_pwd_age * ONE_HUNDRED_NANOSECOND ) / SECONDS_IN_DAY;
 
 		if (max_pwd_age_days <= account->priv->passwd_exp_warn_period) {
-			display_passwd_expiry_message (max_pwd_age_days, 
-						       account);
+			return max_pwd_age_days;
 		}
 	} 
+	return -1;
 }
 
 char *
@@ -1134,10 +1078,10 @@ exchange_account_forget_password (ExchangeAccount *account)
 	e_passwords_forget_password ("Exchange", account->priv->password_key);
 }
 
-#ifdef HAVE_KRB5
 ExchangeAccountResult
 exchange_account_set_password (ExchangeAccount *account, char *old_pass, char *new_pass)
 {
+#ifdef HAVE_KRB5
 	E2kKerberosResult result;
 	char *domain;
 
@@ -1183,8 +1127,24 @@ exchange_account_set_password (ExchangeAccount *account, char *old_pass, char *n
 	}
 	
 	return EXCHANGE_ACCOUNT_PASSWORD_CHANGE_SUCCESS;
-}
+#else
+	g_warning ("exchange_account_set_password: Not implemented (no KRB5)");
+	return EXCHANGE_ACCOUNT_PASSWORD_CHANGE_FAILED;
 #endif
+}
+
+void 
+exchange_account_set_save_password (ExchangeAccount *account, gboolean save_password)
+{
+	account->priv->account->source->save_passwd = save_password;
+}
+
+gboolean 
+exchange_account_is_save_password (ExchangeAccount *account)
+{
+	return account->priv->account->source->save_passwd;
+}
+
 
 /**
  * exchange_account_set_offline:
@@ -1209,7 +1169,7 @@ exchange_account_set_offline (ExchangeAccount *account)
 		account->priv->ctx = NULL;
 	}
 
-	account->priv->account_online = FALSE;
+	account->priv->account_online = OFFLINE_MODE;
 	g_mutex_unlock (account->priv->connect_lock);
 	return TRUE;
 }
@@ -1231,7 +1191,7 @@ exchange_account_set_online (ExchangeAccount *account)
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), FALSE);
 
 	g_mutex_lock (account->priv->connect_lock);
-	account->priv->account_online = TRUE;
+	account->priv->account_online = ONLINE_MODE;
 	g_mutex_unlock (account->priv->connect_lock);
 	
 	return TRUE;
@@ -1248,45 +1208,22 @@ exchange_account_is_offline (ExchangeAccount *account, int *state)
 {
 	g_return_if_fail (EXCHANGE_IS_ACCOUNT (account));
 	
-	*state = account->priv->account_online ? ONLINE_MODE : OFFLINE_MODE;
+	*state = account->priv->account_online;
 }
 
-// SURF : Picked this from gal/util/e-util.c
-/* This only makes a filename safe for usage as a filename.  It still may have shell meta-characters in it. */
-static void
-e_filename_make_safe (gchar *string)
-{
-	gchar *p, *ts;
-	gunichar c;
-	
-	g_return_if_fail (string != NULL);
-	p = string;
-
-	while(p && *p) {
-		c = g_utf8_get_char (p);
-		ts = p;
-		p = g_utf8_next_char (p);
-		if (!g_unichar_isprint(c) || ( c < 0xff && strchr (" /'\"`&();|<>$%{}!", c&0xff ))) {
-			while (ts<p) 	
-				*ts++ = '_';
-		}
-	}
-}
-
-
 static gboolean
 setup_account_hierarchies (ExchangeAccount *account)
 {
 	ExchangeHierarchy *hier, *personal_hier;
 	ExchangeAccountFolderResult fresult;
 	char *phys_uri_prefix, *dir;
-	DIR *d;
-	struct dirent *dent;
-	int offline;
+	GDir *d;
+	const char *dent;
+	int mode;
 
-	exchange_account_is_offline (account, &offline);
+	exchange_account_is_offline (account, &mode);
 
-	if (offline == UNSUPPORTED_MODE)
+	if (mode == UNSUPPORTED_MODE)
 		return FALSE;
 
 	/* Check if folder hierarchies are already setup. */
@@ -1349,13 +1286,12 @@ setup_account_hierarchies (ExchangeAccount *account)
 	g_free (phys_uri_prefix);
 
 	/* Other users' folders */
-	d = opendir (account->storage_dir);
+	d = g_dir_open (account->storage_dir, 0, NULL);
 	if (d) {
-		while ((dent = readdir (d))) {
-			if (!strchr (dent->d_name, '@'))
+		while ((dent = g_dir_read_name (d))) {
+			if (!strchr (dent, '@'))
 				continue;
-			dir = g_strdup_printf ("%s/%s", account->storage_dir,
-					       dent->d_name);
+			dir = g_strdup_printf ("%s/%s", account->storage_dir, dent);
 			hier = exchange_hierarchy_foreign_new_from_dir (account, dir);
 			g_free (dir);
 			if (!hier)
@@ -1363,7 +1299,7 @@ setup_account_hierarchies (ExchangeAccount *account)
 
 			setup_hierarchy_foreign (account, hier);
 		}
-		closedir (d);
+		g_dir_close (d);
 	}
 
 hierarchies_created:
@@ -1378,7 +1314,7 @@ hierarchies_created:
 	
 	fresult = exchange_hierarchy_scan_subtree (personal_hier,
 						   personal_hier->toplevel,
-						   (offline == OFFLINE_MODE));
+						   mode);
 	if (fresult != EXCHANGE_ACCOUNT_FOLDER_OK) {
 		account->priv->connecting = FALSE;
 		return FALSE;
@@ -1390,13 +1326,12 @@ hierarchies_created:
 	fresult = exchange_hierarchy_scan_subtree (
 		account->priv->favorites_hierarchy,
 		account->priv->favorites_hierarchy->toplevel,
-		(offline == OFFLINE_MODE));
+		mode);
 	if (fresult != EXCHANGE_ACCOUNT_FOLDER_OK && 
 	    fresult != EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST) {
 		account->priv->connecting = FALSE;
 		return FALSE;
 	}
-	
 	return TRUE;
 }
 
@@ -1420,30 +1355,31 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	E2kHTTPStatus status;
 	gboolean redirected = FALSE;
 	E2kResult *results;
-	int nresults;
+	int nresults = 0, mode;
 	GByteArray *entryid;
 	const char *timezone;
 	E2kGlobalCatalogStatus gcstatus;
 	E2kGlobalCatalogEntry *entry;
 	E2kOperation gcop;
 	char *user_name = NULL;
-	int offline;
 
 	*info_result = EXCHANGE_ACCOUNT_UNKNOWN_ERROR; 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
 	*info_result = EXCHANGE_ACCOUNT_CONNECT_SUCCESS;
+	exchange_account_is_offline (account, &mode);
 
-	exchange_account_is_offline (account, &offline);
 	g_mutex_lock (account->priv->connect_lock);
 
-	if ((account->priv->connecting) || (offline == OFFLINE_MODE)){
+	if (account->priv->connecting || mode == OFFLINE_MODE) {
 		g_mutex_unlock (account->priv->connect_lock);
-		if (offline == OFFLINE_MODE) {
+		if (mode == OFFLINE_MODE) {
 			setup_account_hierarchies (account);
-
 			*info_result = EXCHANGE_ACCOUNT_OFFLINE;
 		}
+		else {
+			*info_result = EXCHANGE_ACCOUNT_CONNECT_ERROR;
+		}
 		return NULL;
 	} else if (account->priv->ctx) {
 		g_mutex_unlock (account->priv->connect_lock);
@@ -1469,10 +1405,12 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	if (!pword) {
 		account->priv->connecting = FALSE;
 		g_mutex_unlock (account->priv->connect_lock);
+		*info_result = EXCHANGE_ACCOUNT_PASSWORD_INCORRECT;
+		e2k_autoconfig_free (ac);
 		return NULL;
 	}
 
-	e2k_autoconfig_set_password (ac, g_strdup (pword));
+	e2k_autoconfig_set_password (ac, pword);
 
  try_connect_again:
 	account->priv->ctx = e2k_autoconfig_get_context (ac, NULL, &result);
@@ -1488,10 +1426,12 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 			*info_result = EXCHANGE_ACCOUNT_PASSWORD_EXPIRED;
 			account->priv->connecting = FALSE;
 			g_mutex_unlock (account->priv->connect_lock);
+			e2k_autoconfig_free (ac);
 			return NULL;
 		}
 #endif
 		switch (result) {
+
 		case E2K_AUTOCONFIG_AUTH_ERROR:
 			*info_result = EXCHANGE_ACCOUNT_PASSWORD_INCORRECT;
 			e2k_autoconfig_free (ac);
@@ -1530,6 +1470,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 
 		e2k_autoconfig_free (ac);
 		account->priv->connecting = FALSE;
+		account->priv->account_online = OFFLINE_MODE; /* correct? */
 
 		switch (result) {
 		case E2K_AUTOCONFIG_REDIRECT:
@@ -1598,13 +1539,18 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	if (!setup_account_hierarchies (account)) {
 		*info_result = EXCHANGE_ACCOUNT_UNKNOWN_ERROR;
 		g_mutex_unlock (account->priv->connect_lock);
+		if (nresults)
+			e2k_results_free (results, nresults);
 		return NULL; /* FIXME: what error has happened? */
 	}
 
-	/* Find the password expiery peripod and display warning */
-	find_passwd_exp_period(account, entry);
-	
-	/* Check for quota warnings */
+	account->priv->account_online = ONLINE_MODE;
+	account->priv->connecting = FALSE;
+	account->priv->connected = TRUE;
+
+	if (!account->priv->gc)
+		goto skip_quota;
+	/* Check for quota usage */
 	e2k_operation_init (&gcop);
 	gcstatus = e2k_global_catalog_lookup (account->priv->gc, &gcop,
                                             E2K_GLOBAL_CATALOG_LOOKUP_BY_EMAIL,
@@ -1613,9 +1559,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
                                             &entry);	
 	e2k_operation_free (&gcop);
 
-	/* FIXME: quota warnings are not yet marked for translation!! */
-	/* FIXME: warning message should have quota limit value and optionally current
-	 * usage 
+	/* FIXME: warning message should have quota limit value 
 	 */
 	if (gcstatus == E2K_GLOBAL_CATALOG_OK) {
 
@@ -1633,16 +1577,15 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 					account->priv->quota_limit = entry->quota_warn;
 		}
 	}
-	
-	account->priv->connected = TRUE;
-	account->priv->account_online = TRUE;
-	account->priv->connecting = FALSE;
 
+skip_quota:
 	g_signal_connect (account->priv->ctx, "redirect",
 			  G_CALLBACK (context_redirect), account);
 
 	g_signal_emit (account, signals[CONNECTED], 0, account->priv->ctx);
 	g_mutex_unlock (account->priv->connect_lock);
+	if (nresults)
+		e2k_results_free (results, nresults);
 	return account->priv->ctx;
 }
 
@@ -1717,6 +1660,7 @@ exchange_account_get_standard_uri (ExchangeAccount *account, const char *item)
 
 	if (!account->priv->standard_uris)
 		return NULL;
+
 	return g_hash_table_lookup (account->priv->standard_uris, item);
 }
 
@@ -1823,7 +1767,6 @@ exchange_account_get_folder (ExchangeAccount *account,
 
 	if (!path_or_uri)
 		return NULL;
-
 	return g_hash_table_lookup (account->priv->folders, path_or_uri);
 }
 
@@ -1900,8 +1843,13 @@ exchange_account_get_quota_limit (ExchangeAccount *account)
 int
 exchange_account_check_password_expiry (ExchangeAccount *account)
 {
+	E2kGlobalCatalogEntry *entry;
+	int max_pwd_age_days = -1;
+
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), 0);
-	return -1;
+
+	max_pwd_age_days = find_passwd_exp_period (account, entry);
+	return max_pwd_age_days;
 }
 
 char *
@@ -2023,7 +1971,7 @@ ExchangeAccount *
 exchange_account_new (EAccountList *account_list, EAccount *adata)
 {
 	ExchangeAccount *account;
-	char *enc_user, *mailbox, *old_uri_authority;
+	char *enc_user, *mailbox;
 	const char *param, *proto="http", *owa_path, *pf_server, *owa_url; 
 	const char *passwd_exp_warn_period, *offline_sync;
 	E2kUri *uri;
@@ -2055,8 +2003,7 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 
 	/* URI, etc, info */
 	enc_user = e2k_uri_encode (uri->user, FALSE, "@/;:");
-	old_uri_authority = g_strdup_printf ("%s@%s", enc_user,
-							uri->host);
+
 	if (uri->authmech)
 		account->priv->uri_authority = g_strdup_printf ("%s;auth=%s@%s", enc_user,
 								uri->authmech, uri->host);
@@ -2071,9 +2018,13 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 
 	/* Backword compatibility; FIXME, we should just migrate the
 	 * password from this to source_uri.
+	 * old_uri_authority = g_strdup_printf ("%s@%s", enc_user,
+	 *					uri->host);
+	 * old_uri_authority needs to be used in the key for migrating 
+	 * passwords remembered.
 	 */
-	account->priv->password_key = g_strdup_printf ("exchange://%s", old_uri_authority);
-	g_free (old_uri_authority);
+	account->priv->password_key = g_strdup_printf ("exchange://%s/", 
+							account->priv->uri_authority);
 
 	account->priv->username = g_strdup (uri->user);
 	if (uri->domain)
