@@ -22,6 +22,9 @@ extern "C" {
 void 			add_folder_esource (ExchangeAccount *account, FolderType folder_type, const char *folder_name, const char *physical_uri);
 void 			remove_folder_esource (ExchangeAccount *account, FolderType folder_type, const char *physical_uri);
 
+/* Remove this ugly hack by moving this to exchange-account.h */
+char * exchange_account_get_authtype (ExchangeAccount *account);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
